from typing import Any, List
import pandas as pd


class Data_filter:

    def __init__(self, prepared_csv_path : str, filters: dict, filtered_csv_path: str) -> Any : 
        """Initialise the class

        Args:
            prepared_csv_path (str): Prepared csv location path (input)
            filters (dict): Filters to apply to the prepared dataset
            filtered_csv_path (str):  Filtered csv location path (output)

        Returns:
            Any:
        """        
        self.filters = filters
        self.prepared_csv_path = prepared_csv_path
        self.filtered_csv_path = filtered_csv_path

    def load_prepared_df(self) -> pd.DataFrame:
        """Load prepared dataframe

        Returns:
            pd.DataFrame: loaded dataframe
        """        
        prepared_df = pd.read_csv(self.prepared_csv_path)
        return prepared_df

    def get_filter_criterion(self, filters: dict, filter_criterion: str) -> List[Any]:
        """Get a filter criterion values from all filters

        Args:
            filters (dict): all filters
            filter_criterion (str): criterion values to get

        Returns:
            List[Any]: list of values for the particular criterion
        """        
        if filter_criterion in filters:
            return filters[filter_criterion]
        else:
            return []

    def get_filter_values(self, df_to_filter: pd.DataFrame, filter: dict, column: str) -> pd.DataFrame:
        if len(filter) > 0:
            filtered = df_to_filter[df_to_filter[column].isin(filter)]
            return filtered
        else:
            return df_to_filter

    def get_filter_criteria(self) -> List[Any]:
        # get filters criteria
        summoners = self.get_filter_criterion(self.filters, 'summoners')
        picks = self.get_filter_criterion(self.filters, 'picks')
        roles = self.get_filter_criterion(self.filters, 'roles')
        positions = self.get_filter_criterion(self.filters, 'positions')
        seasons = self.get_filter_criterion(self.filters, 'seasons')
        #versions = self.get_filter_criterion(filters,'versions')
        teams = self.get_filter_criterion(self.filters, 'teams')
        return summoners, picks, roles, positions, seasons, teams
    
    def get_filtered_df(self):

        prepared_df = self.load_prepared_df()
        filters = self.filters
        # get filters criteria
        summoners = self.get_filter_criterion(filters, 'summoners')
        picks = self.get_filter_criterion(filters, 'picks')
        roles = self.get_filter_criterion(filters, 'roles')
        positions = self.get_filter_criterion(filters, 'positions')
        seasons = self.get_filter_criterion(filters, 'seasons')
        #versions = self.get_filter_criterion(filters,'versions')
        teams = self.get_filter_criterion(filters, 'teams')

        # ============================================================================
        # Création du "sous-dataset" ne contenant que les picks qui correspondent
        # aux critères spécifiés dans filters
        # ============================================================================
        # filter summoners
        filtered_df = self.get_filter_values(
            prepared_df, summoners, 'summoner_name')
        # filter Champions
        filtered_df = self.get_filter_values(
            filtered_df, picks, 'champion_name')

        # filter roles
        filtered_df = self.get_filter_values(
            filtered_df, roles, 'role')

        # filter position
        filtered_df = self.get_filter_values(
            filtered_df, positions, 'position')

        # filter seasons
        filtered_df = self.get_filter_values(
            filtered_df, seasons, 'game_season')

        # # filter versions
        # filtered_df= self.get_filter_values(filtered_df, versions, 'game_version')

        # filter Teams
        filtered_df = self.get_filter_values(
            filtered_df, teams, 'summoner_team')
        return filtered_df
    
    def get_filtered_csv(self):
        filtered_df = self.get_filtered_df()
        filtered_df.to_csv(self.filtered_csv_path)

filters = {
    'picks': ["Ezreal", "Renekton" ]
    # 'seasons': [10.0, 11.0, 12.0, 13.0],
    # 'teams': ['C9']
    #'positions': ['TOP', 'MIDDLE']
}
 
