from typing import Any
import pandas as pd

from kcorp.data.Data_filter import Data_filter


class Stat_generator:

    def __init__(self, prepared_csv_path: str, summary_csv_path: str, all_params: dict):
        self.prepared_csv_path = prepared_csv_path
        self.summary_csv_path = summary_csv_path
        self.all_params = all_params

    def load_prepared_df(self) -> pd.DataFrame:
        """Load Summoner Dataframe

        Returns:
            pd.DataFrame: dataframe from  a prepared csv file
        """
        prepared_df = pd.read_csv(self.prepared_csv_path)
        return prepared_df

    def get_filtered_df(self, params):
        data_filter = Data_filter(self.prepared_csv_path, params, "")
        return data_filter.get_filtered_df()

    def get_param_criterion(self, all_params: dict, params_criterion: str):
       
        if params_criterion in all_params:
            criterion_values = all_params[params_criterion]
            return criterion_values
        else:
            return []

    def get_stats(self, params, verbose=0):

        prepared_df = self.load_prepared_df()

        # ============================================================================
        # Création du "sous-dataset" ne contenant que les picks qui correspondent
        # aux critères spécifiés dans params
        # ============================================================================
        filtered_df = self.get_filtered_df(params)

        total_games = prepared_df.groupby('game_id').count().shape[0]
        total_filtered_games =  filtered_df.groupby('game_id').count().shape[0]
        picks = self.get_param_criterion(params, 'picks')
        # ============================================================================
        # Calcul des métriques
        # ============================================================================
        # hack time !
        # Step 1 : temp_df = temp_df = filtered_df.groupby(['game_id', 'game_team']).count()
        # =======> On regroupe les picks par equipe et on compte le nombre de lignes que ça représente
        # =======> On récupère des valeurs entre
        # =======>          Minimum 1 :                     un seul champion de la liste est présent dans cette équipe
        # =======>          Maximum len(params['picks']) :  tous les champions de la liste sont présents dans cette équipe
        temp_df = filtered_df.groupby(['game_id', 'game_team', 'win']).count().reset_index()
       

        # Step 2 : pick_nb = temp_df[temp_df['champion_id'] == len(params['picks'])].shape[0]
        # =======> On compte le nombre d'équipes dont le count = nombre de champions de la liste de picks
        # =======> Ca nous donne le nombre de fois ou cet ensemble de champion a été pick
        if len(picks) > 0:
            pick_win_nb = temp_df[(temp_df['win'] == 1.0) & (
                temp_df['champion_id'] == len(picks))].shape[0]
            pick_loose_nb = temp_df[(temp_df['win'] == 0.0) & (
                temp_df['champion_id'] == len(picks))].shape[0]
            pick_total_nb = pick_loose_nb + pick_win_nb
        else:
            pick_win_nb = temp_df[(temp_df['win'] == 1.0)].shape[0]
            pick_loose_nb = temp_df[(temp_df['win'] == 0.0)].shape[0]
            pick_total_nb = pick_loose_nb + pick_win_nb

        summoners = self.get_param_criterion(params, 'summoners')
        champs = self.get_param_criterion(params, 'picks')
        roles = self.get_param_criterion(params, 'roles')
        positions = self.get_param_criterion(params, 'positions')
        seasons = self.get_param_criterion(params, 'seasons')
        #versions = self.get_filter_criterion(all_params,'versions')
        teams = self.get_param_criterion(params, 'teams')
        # ============================================================================
        # Synthèse des résultats
        # ============================================================================
        stats = {

            #'seasons': seasons,
            #'teams': teams,
            'champs': champs,
            'pick_rate': pick_total_nb / total_filtered_games,
            'win_rate': pick_win_nb / pick_total_nb if pick_total_nb > 0 else 0,
            'total_games': total_games,
            'pick_win_nb': pick_win_nb,
            'pick_loose_nb': pick_loose_nb,
            'pick_total_nb': pick_total_nb,
        }
        return stats

    # def set_params(self)-> dict:

    #     """Set champions params

    #     Returns:
    #        dict: dict of parameters to query
    #     """
    #     params = {
    #         'picks' : ['Renekton'], #'Ezreal', 'Renekton', 'Akali', 'Thresh', 'Lee Sin', 'Leona'
    #         'seasons' : [10.0, 11.0, 12.0, 13.0],
    #         'teams' : ['C9'],
    #     }
    #     return params
    def get_picks_size(self):
        pikcs_size= len(self.get_param_criterion(self.all_params, 'picks'))
        return pikcs_size
        

    def get_all_stats(self):
        # all_params = self.all_params
        # summoner_df = self.load_prepared_df()
        # ============================================================================
        # Extract params
        # ============================================================================
        # get all_params criteria
        # summoners = self.get_param_criterion(self.all_params, 'summoners')
        champs = self.get_param_criterion(self.all_params, 'picks')
        # roles = self.get_param_criterion(self.all_params, 'roles')
        # positions = self.get_param_criterion(self.all_params, 'positions')
        # seasons = self.get_param_criterion(self.all_params, 'seasons')
        # #versions = self.get_filter_criterion(all_params,'versions')
        # teams = self.get_param_criterion(self.all_params, 'teams')
        # max_tuple_size = self.get_param_criterion(self.all_params, 'max_tuple_size')

        #get picks size
        picks_size = self.get_picks_size()

        # ============================================================================
        # Parcours des différents combinaisons
        # ============================================================================
        picks=champs
        params = {
                    'picks': picks,}
        result = self.get_stats(params)
        return result

    def get_summary_csv(self):
        """Get summary Csv file
        """
        summary_list = self.get_all_stats()
        summary_df = pd.DataFrame(summary_list)
        summary_df.to_csv(self.summary_csv_path)



